#! /usr/bin/perl

push(@INC, grep(-f "$_/tfm.pl",split(':',$ENV{'PATH'})));
require 'tfm.pl';

# Pieces:
#   g1  horizon
#   g2  eq circles
#   g3  eq stars
#   g4  ecliptic
#   g5  milky way

# Equatorial-to-altazimuth tfm.
# Altazimuth has +Z zenith, +X east, +Y north.
$LST = 5*15;
$LAT = 40;
@Tea = &m3( &tmul( &tfm('z', 270-$LST), &tfm('x', $LAT-90) ) );
@Tae = &transpose(@Tea);

$Tea = join(" ", @Tea);
$Tae = join(" ", @Tae);

$Tze = join(" ", @Tze);
$Tge = join(" ", @Tge);

@circle = ();
# Compute points on a great circle.
for($az = 0; $az <= 360; $az += 5) {
  push(@circle, sprintf("%.5f %.5f %.5f\n", cos(&rad($az)), sin(&rad($az)), 0));
}
$circle = join("", "73 1\n", @circle);

# Draw a bunch of small circles in equatorial system,
# omitting points below the horizon.
for($dec = 90; $dec >= -90; $dec -= 10) {
    $cdec = cos(&rad($dec));
    $sdec = sin(&rad($dec));
    $nu = 144;
    for($u = 0; $u < $nu; $u++) {
	$har = $u/$nu * 2*$pi;
	@pe = (	cos($har)*$cdec,
		sin($har)*$cdec,
		$sdec );

	# Is it above the horizon?  Z>=0?
	@pa = &vm3mul( @pe, @Tea );
	## next if $pa[2]<0;
	push(@eqcircs, sprintf("%.4f %.4f %.4f  %d\n", @pe, $pa[2]<0 ? 1 : 0));
	next if $dec == 90 || $dec == -90;
    }
}
$eqcircs = join("", @eqcircs);

print <<EOF;
object g1=horizon
eval lsize .125
eval label on
eval laxes off
eval alpha 1

tfm $Tae

mesh -c 1 -s solid {
2 2
-50 -50 -.005
50 -50 -.005
-50 50 -.005
50 50 -.005
}

# Horizon circle
mesh -c 0 -s line {
$circle
}

0 0 1.1    text Zenith
1.2 0 .1    text E
-1.2 0 .1   text W
0 1.2 .1    text N
0 -1.2 .1   text S

cment 0  1 0 0
cment 1  0 0 0
cment 2  .3 .3 .7

0 0 0 ellipsoid -s line -r 0,0,1 -n 2,2  -c 2

object g2=eqcircs

datavar 0  color
eval lum const 10
eval color color 0
eval color color exact
eval cment 0  .5 .5 .2  # Above horizon
eval cment 1  .2 .2 .5  # Below horizon
eval cment 2  0 0 .45   # Below horizon


0 0 0 text
eval lsize 1
eval label on
eval laxes off

1 0 0 text -size .125 0h
.707 .707 0 text -size .125 3h
0 1 0 text -size .125 6h
-.707 .707 0 text -size .125 9h
-1 0 0 text -size .125 12h
-.707 -.707 0 text -size .125 15h
0 -1 0 text -size .125 18h
.707 -.707 0 text -size .125 21h

$eqcircs

object g3=stars

read hipbrightest.speck
eval cmap colorbv.cmap
eval color color -.5 2
eval lsize 0.5
eval labels on
eval labelsize 1
eval laxes off

eval lum lum 0 1
eval psize 10000
eval slum 30

eval jump -1.88504 -0.230048 8.23578  3.20152 -13.517 -22.4808
eval winsize 560x560

object g4=ecliptic

tfm $Tze

eval cment 0  0 1 0
mesh -c 0 -s line {
$circle
}

object g5=galactic

eval cment 0  0 .5 1
tfm $Tge
mesh -c 0 -s line {
$circle
}

EOF

